@@ -166,6 +166,13 @@ static void LOG_DEBUG(const char * fmt, ...)
      va_end(ap);
 }
 
+/**
+ * Take the standard printf type arguments and write to stdout if verbose is
+ * set.
+ *
+ * @param[in] fmt The printf format args
+ * @param[in] ... The printf style arguments
+ */
 static void LOG_INFO(const char * fmt, ...)
 {
      va_list ap;
@@ -174,6 +181,13 @@ static void LOG_INFO(const char * fmt, ...)
      va_end(ap);
 }
 
+/**
+ * Take the standard printf type arguments and write to stdout if verbose is
+ * set.
+ *
+ * @param[in] fmt The printf format args
+ * @param[in] ... The printf style arguments
+ */
 static void NOTICE(const char * fmt, ...)
 {
      va_list ap;
@@ -183,7 +197,13 @@ static void NOTICE(const char * fmt, ...)
      va_end(ap);
 }
 
-
+/**
+ * Take the standard printf type arguments and write to stdout if verbose is
+ * set.
+ *
+ * @param[in] fmt The printf format args
+ * @param[in] ... The printf style arguments
+ */
 static void WARNING(const char * fmt, ...)
 {
      va_list ap;
@@ -193,6 +213,13 @@ static void WARNING(const char * fmt, ...)
      va_end(ap);
 }
 
+/**
+ * Take the standard printf type arguments and write to stdout if verbose is
+ * set.
+ *
+ * @param[in] fmt The printf format args
+ * @param[in] ... The printf style arguments
+ */
 static void ERROR(const char * fmt, ...)
 {
      va_list ap;
@@ -204,7 +231,7 @@ static void ERROR(const char * fmt, ...)
 }
 
 /**
- * Check the path exists and if so allocate memory to hold the path name.
+ * Check the path exists and if so allocate memory to hold the path name
  *
  * @param[in] fname full path to application
  *
@@ -358,7 +385,7 @@ static void del_cache_entry(appCacheEntry_t * entry)
 /**
  * Delete the application cache
  */
-static void delete_cache()
+static void delete_cache(void)
 {
      while(g_cache)
      {
@@ -819,6 +846,8 @@ static handler_t * create_handler(pluginType_t * plugin)
  * Read the configuration file into memory.
  *
  * @param[in] f The FILE pointer
+ *
+ * @return A allocated and filled in pluginType_t
  */
 static pluginType_t * read_config(FILE * f)
 {
@@ -880,11 +909,15 @@ static pluginType_t * read_config(FILE * f)
                          /* Add to end of linked list of handlers */
                          prev_cmd->pNext = cmd;
                     }
-                    else
+                    else if(handler)
                     {
                          /* Add at header of link list of handlers */
                          handler->cmds = cmd;
                     }
+                    else
+                    {
+                         ERROR("Unexpected command before mimetypes '%s'\n", buffer);
+                    }
                     /* Remember the current end of linked list */
                     prev_cmd = cmd;
                }
@@ -894,14 +927,14 @@ static pluginType_t * read_config(FILE * f)
 }
 
 
-
 /**
  * Read config callback function. Open and run the configuration file (fname)
  * through M4 and then call read_config() to parse the output
  *
  * @param[in] fname Full name (and path) of configuration file
+ * @param[out] m4_pid The pid of the m4 process
  *
- * @return 1(true) if success, 0(false) if not
+ * @return Open file to read config from
  */
 static FILE * open_config(const char * fname, pid_t * m4_pid)
 {
@@ -958,6 +991,9 @@ static FILE * open_config(const char * fname, pid_t * m4_pid)
 
 /**
  * Close the config file
+ *
+ * @param[in] fp The config file object
+ * @param[in] m4_pid The PID of the m4 process
  */
 static void close_config(FILE * fp, pid_t m4_pid)
 {
@@ -982,7 +1018,7 @@ static void close_config(FILE * fp, pid_t m4_pid)
  *
  * @param[in] plugibName The name of the plugin
  *
- * @return none
+ * @return A linked list of pluginTypes
  */
 static pluginType_t * do_read_config(const char * config_fname)
 {
@@ -1004,6 +1040,8 @@ static pluginType_t * do_read_config(const char * config_fname)
 
 /**
  * Delete a type
+ *
+ * @param[in] type The mimetype_t structure
  */
 static void delete_type(mimetype_t * type)
 {
@@ -1014,6 +1052,8 @@ static void delete_type(mimetype_t * type)
 
 /**
  * Delete a command
+ *
+ * @param[in] cmd The command_t structure
  */
 static void delete_cmd(command_t * cmd)
 {
@@ -1026,6 +1066,8 @@ static void delete_cmd(command_t * cmd)
 
 /**
  * Delete a handler
+ *
+ * @param[in] handler The handler_t structure
  */
 static void delete_handler(handler_t * handler)
 {
@@ -1047,6 +1089,8 @@ static void delete_handler(handler_t * handler)
 
 /**
  * Delete a plugin
+ *
+ * @param[in] plugin The pluginType_t structure
  */
 static void delete_plugin(pluginType_t * plugin)
 {
@@ -1062,6 +1106,13 @@ static void delete_plugin(pluginType_t * plugin)
      free(plugin);
 }
 
+/**
+ * Trim the linked list of plugins
+ *
+ * @param[in] plugins The linked list of plugins
+ *
+ * @return The new linked list of plugins
+ */
 static pluginType_t * trim(pluginType_t * plugins)
 {
     pluginType_t * prev_plugin = NULL; /* Skip the default */
@@ -1120,40 +1171,60 @@ static pluginType_t * trim(pluginType_t * plugins)
     return plugins;
 }
 
-static void write_type(mimetype_t * type, FILE * fp1, FILE * fp2)
+/**
+ * Write the mimetype to the mimetypes and cmds files
+ *
+ * @param[in] type Pointer to the mimetype structure.
+ * @param[in] mimetypes_fp Pointer to the mimetypes file
+ * @param[in] cmds_fp Pointer to the cmds file
+ */
+static void write_type(mimetype_t * type, FILE * mimetypes_fp, FILE * cmds_fp)
 {
      LOG_DEBUG("%s\n", type->type);
-     fprintf(fp1, "%s:%s;", type->type, type->desc);
-     fprintf(fp2, "%s\n", type->type);
+     fprintf(mimetypes_fp, "%s:%s;", type->type, type->desc);
+     fprintf(cmds_fp, "%s\n", type->type);
 }
 
+/**
+ * Write the cmd to the cmds files
+ *
+ * @param[in] cmd Pointer to the cmd structure
+ * @param[in] cmds_fp Pointer to the cmds file
+ */
 static void write_cmd(command_t * cmd, FILE * fp)
 {
      LOG_DEBUG("\t0x%x\t%s\t%s:%s\n", cmd->flags, cmd->winname, cmd->fmatchStr, cmd->cmd);
      fprintf(fp, "\t%x\t%s\t%s\t%s\n", cmd->flags, cmd->winname ? cmd->winname : "", cmd->fmatchStr ? cmd->fmatchStr : "", cmd->cmd);
 }
 
-static void write_handler(handler_t * handler, FILE * fp1, FILE * fp2)
+/**
+ * Write the handler to the mimetypes and cmds files
+ *
+ * @param[in] handler Pointer to the handler structure.
+ * @param[in] mimetypes_fp Pointer to the mimetypes file
+ * @param[in] cmds_fp Pointer to the cmds file
+ */
+static void write_handler(handler_t * handler, FILE * mimetypes_fp, FILE * cmds_fp)
 {
      mimetype_t * type = handler->types;
      command_t * cmd = handler->cmds;
 
      for(; type; type = type->pNext)
      {
-          write_type(type, fp1, fp2);
+          write_type(type, mimetypes_fp, cmds_fp);
      }
      for(; cmd; cmd = cmd->pNext)
      {
-          write_cmd(cmd, fp2);
+          write_cmd(cmd, cmds_fp);
      }
 }
 
 /**
  * Get the directory where processed config files are put
  *
- * return allocated string containing the path
+ * @return allocated string containing the path
  */
-char * getProcessedDir()
+static char * get_processed_dir(void)
 {
      char path[MAX_FILE_PATH_LEN];
      const char * home = getenv("HOME");
@@ -1192,15 +1263,28 @@ char * getProcessedDir()
 
 #define BUF_LEN (1024)
 
-/******************************************************************************/
-size_t maxStringLen(const char * str1, const char * str2)
+/**
+ * Return the length of the longest string
+ *
+ * @param[in] str1 String 1
+ * @param[in] str2 String 2
+ *
+ * @return The max length
+ */
+static size_t max_string_len(const char * str1, const char * str2)
 {
     const size_t len1 = strlen(str1);
     const size_t len2 = strlen(str2);
     return (len1 > len2) ? len1 : len2;
 }
 
-/******************************************************************************/
+/**
+ * Copy mozplugger.so from in to out and add in the magic value
+ *
+ * @param[in] in filename of source
+ * @param[in] out filename of destination
+ * @param[in] magic The magic string
+ */
 static void copy(const char * in, const char * out, const char * magic)
 {
     char buffer[BUF_LEN];
@@ -1222,7 +1306,7 @@ static void copy(const char * in, const char * out, const char * magic)
         ERROR("Failed to open '%s'\n", out);
     }
 
-    extra = maxStringLen(PLACE_HOLDER_STR, magic)+1;
+    extra = max_string_len(PLACE_HOLDER_STR, magic)+1;
     len = fread(buffer, 1, BUF_LEN, inFp);
 
     while(1)
@@ -1265,16 +1349,21 @@ static void copy(const char * in, const char * out, const char * magic)
  * @param[in] pluginIdx The plugin idx number
  *
  */
-void install(const char * localPluginDir, int cfgIdx, int pluginIdx)
+static void install(const char * localPluginDir, int cfgIdx, int pluginIdx)
 {
     char localPluginPath[MAX_FILE_PATH_LEN];
+    char tmpPluginPath[MAX_FILE_PATH_LEN];
     char globalPluginPath[MAX_FILE_PATH_LEN];
     char magic[MAX_PLUGIN_MAGIC_LEN];
     char * sep;
 
+    snprintf(tmpPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger.tmp", localPluginDir);
     snprintf(localPluginPath, MAX_FILE_PATH_LEN, "%s/mozplugger%i.so", localPluginDir, pluginIdx);
     snprintf(magic, MAX_PLUGIN_MAGIC_LEN, "%i:", cfgIdx);
 
+    unlink(tmpPluginPath);
+    rename(localPluginPath, tmpPluginPath);
+
     strncpy(globalPluginPath, GLOBAL_PLUGIN_DIRS, sizeof(globalPluginPath));
     if( (sep = strchr(globalPluginPath, ' ')) == NULL)
     {
@@ -1319,6 +1408,12 @@ static void write_helpers(FILE * fp)
      }
 }
 
+/**
+ * Remove any obsolete plugins in local plugin directory
+ *
+ * @param[in] localPluginDir Path to local plugin directory
+ * @param[in] idx The first obsolete plugin to delete
+ */
 static void removeRest(const char * localPluginDir, int idx)
 {
      char pluginPath[MAX_FILE_PATH_LEN];
@@ -1357,7 +1452,7 @@ static void write_plugin(const char * config_fname, const char * localPluginDir,
      FILE * fp2;
      handler_t * handler;
      char buffer[MAX_FILE_PATH_LEN];
-     char * path = getProcessedDir();
+     char * path = get_processed_dir();
 
      install(localPluginDir, cfgIdx, pluginIdx);
 
@@ -1368,7 +1463,7 @@ static void write_plugin(const char * config_fname, const char * localPluginDir,
      {
           ERROR("Failed to open '%s'\n", buffer);
      }
-     fprintf(fp1, "#%s\n", PROC_CONF_FMT);
+     fprintf(fp1, "#%s\n", VERSION);
      fprintf(fp1, "# This is autogenerated from %s\n", config_fname);
      fprintf(fp1, "%s\t%s\n", "name", plugin->name);
      fprintf(fp1, "%s\t%s\n", "version", plugin->version);
@@ -1380,7 +1475,7 @@ static void write_plugin(const char * config_fname, const char * localPluginDir,
      {
           ERROR("Failed to open '%s'\n", buffer);
      }
-     fprintf(fp1, "#%s\n", PROC_CONF_FMT);
+     fprintf(fp1, "#%s\n", VERSION);
      fprintf(fp1, "# This is autogenerated from %s\n", config_fname);
 
      snprintf(buffer, sizeof(buffer), "%s/%i.cmds", path, cfgIdx);
@@ -1389,7 +1484,7 @@ static void write_plugin(const char * config_fname, const char * localPluginDir,
           fclose(fp1);
           ERROR("Failed to open '%s'\n", buffer);
      }
-     fprintf(fp2, "#%s\n", PROC_CONF_FMT);
+     fprintf(fp2, "#%s\n", VERSION);
      fprintf(fp2, "# This is autogenerated from %s\n", config_fname);
 
      for(handler = plugin->handlers; handler; handler = handler->pNext)
@@ -1468,6 +1563,11 @@ int main(int argc, char * argv[])
      int i;
      int cfgIndex = 0;
 
+     if((argc > 1) && (strcmp(argv[1], "-v") == 0))
+     {
+          g_verbose = 1;
+     }
+
      if(home == NULL)
      {
           ERROR("HOME not defined\n");
