@@ -1071,7 +1071,7 @@ static int match_url(const char * matchStr, const char * url)
 __inline
 static int match_command(const data_t * THIS, int streamOnly, const command_t *c)
 {
-#define MODE_MASK (H_NOEMBED | H_EMBED | H_LINKS)
+#define MODE_MASK (H_NOEMBED | H_EMBED)
 
      D("Checking command: %s\n", c->cmd);
 
@@ -1086,6 +1086,14 @@ static int match_command(const data_t * THIS, int streamOnly, const command_t *c
 	       return 0;
           }
      }
+     if((THIS->mode_flags & H_LINKS) != 0)   /* Requires the links helper? */
+     {
+          if((c->flags & MODE_MASK) == 0)    /* But command is not */
+          {
+	       D("Flag mismatch: cmd doesnt do links\n");
+	       return 0;
+          }
+     }
 
      if ((c->flags & H_LOOP) && (THIS->repeats != INF_LOOPS))
      {
@@ -1241,6 +1249,7 @@ static bool mozplugger_update(void)
      else
      {
          int status;
+         D("Waiting for mozplugger-update\n");
          waitpid(pid, &status, 0);          /* If Application completed is a bad way, then lets give up now */
          if(!WIFEXITED(status))
          {
@@ -1253,14 +1262,15 @@ static bool mozplugger_update(void)
                success = false;
          }
      }
+     D("mozplugger-update done\n");
      if(success)
      {
-         NPN_ReloadPlugins(false);
+//         NPN_ReloadPlugins(false);
      }
      return success;
 }
 
-static bool chkValidLocalPluginDirs()
+static bool chkValidLocalPluginDirs(void)
 {
      static const char * browsers[] =
      {
@@ -1298,6 +1308,84 @@ static bool chkValidLocalPluginDirs()
      return true;
 }
 
+/**
+ * Skip over the characters to end marker
+ *
+ * @return the line end marker
+ */
+static char * skip_to_end(char * p)
+{
+     while((*p != '\r') && (*p != '\n') && (*p != '\0'))
+     {
+          p++;
+     }
+     return p;
+}
+
+/**
+ * Skip end marker
+ *
+ * @return the start of next line
+ */
+static char * skip_end(char * p)
+{
+     while((*p == '\r') || (*p == '\n'))
+     {
+          p++;
+     }
+     return p;
+}
+
+/**
+ * Parse buf for text containing the VERSION of the config file. Check the
+ * version matches
+ *
+ * @return NULL if not newer, else pointer to next line in buf
+ */
+static char * chk_version_matches(char * buf)
+{
+     char * next = skip_to_end(buf);
+     *next = '\0';
+     next = skip_end(next + 1);
+     D("Processed config version = '%s'\n", &buf[1]);
+     if(strcmp(&buf[1],  VERSION) != 0)
+     {
+          D("Processed config format mismatch should be" VERSION "\n");
+          next = NULL;
+     }
+     return next;
+}
+ 
+/**
+ * Parse buf for text containing the name of the config file. Check the
+ * date and time of that file against the time of the cached file.
+ *
+ * @return NULL if not newer, else pointer to next line in buf
+ */
+static char * chk_cached_is_newer(char * buf, time_t cached_ftime)
+{
+     char * q = strstr(buf, "autogenerated from ");
+     if(q)
+     {
+          struct stat details;
+          char * next;
+          q += 19;
+          next = skip_to_end(q);
+          *next = '\0';
+          next = skip_end(next + 1);
+
+          if( (stat(q, &details) == 0) && (details.st_mtime > cached_ftime))
+          {
+               buf = NULL;
+          }
+          else
+          {
+               buf = next;
+          }
+    }
+    return buf;
+}
+
 /**
  * Construct a MIME Description string for netscape so that mozilla shall know
  * when to call us back.
@@ -1327,47 +1415,57 @@ const char * NP2_GetMIMEDescription(const char * magic)
      D("Reading '%s'\n", fname);
      if(stat(fname, &details) == 0)
      {
- #ifdef AUTO_UPDATE
-          time_t old;
-          time_t now;
-#endif
+          time_t cached_ftime = details.st_mtime;
           int fd = open(fname, O_RDONLY);
-          if(fd >= 0)
-          {
-               if(magic[0] != '-')
-               {
-                    char * buf = malloc(details.st_size + 1);
-                    if(buf)
-                    {
-                         int bytes = read(fd, buf, details.st_size);
-                         buf[bytes] = '\0';
-
-                         D("Processed config version = '%.4s'\n", &buf[1]);
-                         if(strncmp(&buf[1],  PROC_CONF_FMT, 4) != 0)
-                         {
-                              D("Processed config format mismatch\n");
-                              free(buf);
-#ifdef AUTO_UPDATE
-                              update = true;
-#endif
-                         }
-                         else
-                         {
-                              desc = strrchr(buf, '\n');
-                         }
-                    }
-               }
-               close(fd);
-          }
-
-#ifdef AUTO_UPDATE
-          old = details.st_mtime;
-          now = time(NULL);
-          if((old < now) && (now-old > 7*24*60*60))
+ #ifdef AUTO_UPDATE
+          time_t now = time(NULL);
+          D("Auto update %lu - %lu\n", now, cached_ftime); 
+          if((cached_ftime < now) && (now - cached_ftime > 7*24*60*60))
           {
               update = true;
           }
 #endif
+          if(fd >= 0)
+          {
+               char * buf = malloc(details.st_size + 1);
+               char * p = buf;
+               if(buf)
+               {
+                    int bytes = read(fd, buf, details.st_size);
+                    buf[bytes] = '\0';
+
+                    p = chk_version_matches(buf);
+                    if(!p)
+                    {
+                         update = true;
+                         free(buf);
+                         buf = NULL;
+                    }
+                    else
+                    {
+                         p = chk_cached_is_newer(p, cached_ftime);
+                         if(!p)
+                         {
+                              update = true;
+                              free(buf);
+                              buf = NULL;
+                         }
+                    }
+               }
+               close(fd);
+               if(buf && (magic[0] != '-'))
+               {
+                    while(*p == '#')
+                    {
+                        p = skip_end(skip_to_end(p));
+                    }
+                    desc = p;
+               }
+               else
+               {
+                    free(buf);
+               }
+          }
      }
      else
      {
@@ -1453,6 +1551,10 @@ static const char * getPluginDescription(const char * magic)
      }
      else
      {
+          struct stat details;
+          details.st_mtime = 0;
+          stat(config_fname, &details);
+
           config_fname[strlen(config_fname)-4] = '\0';
 
           snprintf(desc_buffer, sizeof(desc_buffer),
@@ -1462,11 +1564,13 @@ static const char * getPluginDescription(const char * magic)
                    "(gcov)"
 #endif
 		   ", for documentation on mozplugger see the man page. "
+                   "Current cached config created on %s."
 		   "<table>"
 		   "<tr><td>Configuration files:</td><td>%s*</td></tr>"
                    "%s%s%s"
 		   " </table>"
 		   "<br clear=all>",
+                   asctime(localtime(&details.st_mtime)),
 		   config_fname ? config_fname : "Not found!",
                    dbgPath ? "<tr><td>Debug file:</td><td>" : "",
                    dbgPath ? dbgPath : "",
@@ -1484,32 +1588,32 @@ static const char * getPluginDescription(const char * magic)
  */
 static NPBool getPluginNeedsXembed(NPP instance, NPError *pErr)
 {
-     data_t * this;
+     NPBool retVal = 0;
 
      if (instance == NULL)
      {
           *pErr = NPERR_GENERIC_ERROR;
-          return 0;
-     }
-
-     this = instance->pdata;
-     if((this == NULL) || (this->command == NULL))
-     {
-          *pErr = NPERR_GENERIC_ERROR;
-          return 0;
-     }
-
-     if( ((this->command->flags & H_NEEDS_XEMBED) != 0)
-                                              && does_browser_support_xembed())
-     {
-          D("Plugin needs XEmbed\n");
-          return 1;
      }
      else
      {
-          D("Plugin does not need XEmbed\n");
-          return 0;
+          data_t * this = instance->pdata;
+          if((this == NULL) || (this->command == NULL))
+          {
+               *pErr = NPERR_GENERIC_ERROR;
+          }
+
+          else if( ((this->command->flags & H_NEEDS_XEMBED) != 0)
+                                              && does_browser_support_xembed())
+          {
+               D("Plugin needs XEmbed\n");
+               retVal = 1;
+          }
+          else
+          {
+               D("Plugin does not need XEmbed\n");
+          }
      }
+     return retVal;
 }
 
 /**
